# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
import urllib
from types import ListType, StringType
from time import time

# zope imports
from Globals import DTMLFile, InitializeClass, ImageFile
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.Permissions import view
from AccessControl.Role import RoleManager
from Products.PageTemplates import ZopePageTemplate
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from Products.PageTemplates.Expressions import getEngine
from OFS.Image import Image
from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from OFS.ObjectManager import REPLACEABLE
from DocumentTemplate.DT_Util import html_quote

# foreign product imports
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport

# own imports
from Renderable import Renderable
from Utility import getTemplateFile
from Const import *

componentClasses = []

manage_addFieldComponentForm = getTemplateFile('manage_addFieldComponentForm', globals())
manage_addFieldRowComponentForm = getTemplateFile('manage_addFieldRowComponentForm', globals())
manage_addFieldColumnComponentForm = getTemplateFile('manage_addFieldColumnComponentForm', globals())
manage_addLabelComponentForm = getTemplateFile('manage_addLabelComponentForm', globals())
manage_addButtonComponentForm = getTemplateFile('manage_addButtonComponentForm', globals())
manage_addErrorComponentForm = getTemplateFile('manage_addErrorComponentForm', globals())
manage_addErrorColumnComponentForm = getTemplateFile('manage_addErrorColumnComponentForm', globals())

def manage_addFieldComponent(self, field_id, title='', css_class='', valid_for=TEMPLATE_TYPES, REQUEST=None):
    '''manage_addFieldComponent(self, field_id, title='', css_class='', valid_for=TEMPLATE_TYPES, REQUEST=None);
       return None or HTML'''

    component = FieldComponent(field_id, title=title, css_class=css_class, valid_for=valid_for)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addFieldRowComponent(self, field_ids, title='', css_class='', valid_for=TEMPLATE_TYPES, REQUEST=None):
    '''manage_addFieldRowComponent(self, field_ids, title='', css_class='', valid_for=TEMPLATE_TYPES, REQUEST=None);
       return None or HTML'''

    if not ListType is type(field_ids): field_ids = [field_ids]
    for field_id in field_ids:
        component = FieldRowComponent(field_id, title=title, css_class=css_class, valid_for=valid_for)
        self._setObject(component.id, component)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')

def manage_addLabelComponent(self, id, title='', text='', css_class='', valid_for=TEMPLATE_TYPES,
                             label_type=LABEL_TYPE_PLAIN, REQUEST=None):
    '''manage_addLabelComponent(self, id, title='', text='', css_class='', valid_for=TEMPLATE_TYPES,
                             label_type=LABEL_TYPE_PLAIN, REQUEST=None);
       return None or HTML'''

    component = LabelComponent(id, title=title, text=text, label_type=label_type, css_class=css_class,
                               valid_for=valid_for)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addFieldColumnComponent(self, id, title='', text='', css_class='', valid_for=TEMPLATE_TYPES, REQUEST=None):
    '''manage_addFieldColumnComponent(self, id, text, css_class='', REQUEST=None);
       return None or HTML'''

    component = FieldColumnComponent(id, text=text, title=title, css_class=css_class, valid_for=valid_for)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addButtonComponent(self, id, title='', css_class='', action='', button_type=BUTTON_TYPE_SUBMIT, REQUEST=None):
    '''manage_addButtonComponent(self, id, text, css_class='', REQUEST=None);
       return None or HTML'''

    component = ButtonComponent(id, title=title, css_class=css_class, action=action, button_type=button_type)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addErrorComponent(self, id, field_ids=[], title='', css_class='', REQUEST=None):
    '''manage_addErrorComponent(self, field_ids, REQUEST=None);
       return None or HTML'''

    component = ErrorComponent(id, title=title, css_class=css_class, field_ids=field_ids)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

def manage_addErrorColumnComponent(self, id, title='', css_class='', REQUEST=None):
    '''manage_addErrorComponent(self, field_ids, REQUEST=None);
       return None or HTML'''

    component = ErrorColumnComponent(id, title=title, css_class=css_class)
    self._setObject(component.id, component)
    ob = self._getOb(component.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

class AbstractComponent(SubObjectTypeSupport, Folder, Renderable):
    security = ClassSecurityInfo()
    selected_meta_types = ('Page Template',)
    manage_options = Folder.manage_options
    manage_workspace = Folder.manage_propertiesForm
    
    _properties = Folder._properties + \
                  ({'id': 'css_class', 'type': 'string', 'mode':'w', 'default': ''},
                   {'id': 'valid_for', 'type': 'multiple selection', 'mode':'w',
                    'default':TEMPLATE_TYPES, 'select_variable':'getTemplateTypes'}
                   )
    

    def __init__(self, id, title='', css_class='', valid_for=[]):
        self.id = str(id)
        self.title = title
        self.css_class = css_class
        if StringType is type(valid_for):
            self.valid_for = [valid_for]
        else:
            self.valid_for = tuple(valid_for)

    _form_template = None
    _report_template = None
    
InitializeClass(AbstractComponent)

class FieldComponent(AbstractComponent):
    meta_type = 'Field Component'
    security = ClassSecurityInfo()
    icon_broken = 'misc_/FormulatorLayout/FieldComponent_broken'

    _properties = AbstractComponent._properties + \
                  ({'id': 'field_id', 'type': 'string', 'mode':'r', 'default': ''},
                   )

    def __init__(self, field_id, title='', css_class='', valid_for=[]):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        FieldComponent.inheritedAttribute('__init__')(self, field_id, title, css_class, valid_for)
        self.id = field_id
        self.field_id = field_id

    security.declareProtected(view, 'getField')
    def getField(self):
        form = self.getForm()
        try:
            field = form[self.field_id]
        except:
            raise 'Broken Field'
        return field

    security.declarePrivate('isBroken')
    def isBroken(self):
        try:
            self.getField()
        except:
            return 1
        return 0

    security.declarePrivate('getBoundNames')
    def getBoundNames(self, template_type=TEMPLATE_TYPE_DEFAULT, **kw):
        bound_names = FieldComponent.inheritedAttribute('getBoundNames') \
                      (self, template_type=template_type, **kw)
        ob = self.getRequestOption('ob')
        custom_getattr = self.getRequestOption('custom_getattr')
        field = self.getField()
        field_value = None
        if field is not None and ob is not None:
            if template_type == TEMPLATE_TYPE_FORM:
                field_value = self.getEditableValue(ob, field, custom_getattr=custom_getattr)
            else:
                field_value = self.renderFieldStatic(ob, field, custom_getattr=custom_getattr)
        bound_names.update({'field': field,
                            'field_value': field_value})
        return bound_names

    security.declarePrivate('getFieldIds')
    def getFieldIds(self):
        return [self.field_id]

    security.declarePrivate('isValidFor')
    def isValidFor(self, template_type):
        is_valid = FieldComponent.inheritedAttribute('isValidFor')(self, template_type)
        if is_valid:
            field = self.getField()
            if template_type == TEMPLATE_TYPE_FORM:
                if not field.get_value('hidden'):
                    return 1
            elif template_type == TEMPLATE_TYPE_REPORT:
                ob = self.getRequestOption('ob')
                custom_getattr = self.getRequestOption('custom_getattr')
                omit_empty = self.getRequestOption('omit_empty', 1)
                return self.isFieldValidForOb(ob, field, custom_getattr, omit_empty)
        return 0
    
    security.declareProtected(view, 'icon')
    def icon(self):
        try:
            icon =  self.getField().__class__.icon
            if callable(icon):
                icon = icon()
        except:
            icon = FieldComponent.icon_broken
        return icon

    _form_template = getTemplateFile('templates/FieldComponent')
    _report_template = getTemplateFile('templates/FieldComponent_report')
        
InitializeClass(FieldComponent)
componentClasses.append(FieldComponent)

class LabelComponent(ZopePageTemplate.ZopePageTemplate, AbstractComponent):
    meta_type = 'Label Component'
    _icon = 'misc_/FormulatorLayout/LabelComponent'
    _icon_broken = 'misc_/FormulatorLayout/LabelComponent_broken'
    security = ClassSecurityInfo()

    _properties = AbstractComponent._properties + (
        {'id':'label_type', 'type':'selection', 'mode':'w',
         'default':LABEL_TYPE_PLAIN, 'select_variable':'getLabelTypes'},
        )

    def __init__(self, id, text='', title='', css_class='', valid_for=[], label_type=LABEL_TYPE_PLAIN):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        AbstractComponent.__init__(self, id, title=title, css_class=css_class, valid_for=valid_for)
        ZopePageTemplate.ZopePageTemplate.__init__(self, id, text=text)
        self.label_type = label_type

    security.declareProtected(view, 'render')
    def render(self, **kw):
        REQUEST = self.REQUEST
        if self.label_type == LABEL_TYPE_PAGE_TEMPLATE:
            bound_names = self.getBoundNames()
            bound_names.update(kw)
            return self.pt_render(extra_context=bound_names)
        else:
            text = self.read()
            if self.label_type == LABEL_TYPE_PLAIN:
                return html_quote(text)
            else:
                return text

    security.declareProtected(view, 'icon')
    def icon(self):
        if self._v_errors:
            return self._icon_broken
        else:
            return self._icon
        
        
InitializeClass(LabelComponent)
componentClasses.append(LabelComponent)

class FieldRowComponent(FieldComponent):
    meta_type = 'Field Row Component'
    security = ClassSecurityInfo()
    
    _form_template = getTemplateFile('templates/FieldRowComponent')
    _report_template = getTemplateFile('templates/FieldRowComponent_report')
        
InitializeClass(FieldRowComponent)
componentClasses.append(FieldRowComponent)

class FieldColumnComponent(LabelComponent):
    meta_type = 'Field Column Component'
    icon = 'misc_/FormulatorLayout/FieldColumn'
    security = ClassSecurityInfo()
    _properties = AbstractComponent._properties + ({'id':'width', 'type':'string', 'mode':'w', 'default':''},
                                                   {'id':'label_type', 'type':'string', 'mode':'r', 'default':''})

    def __init__(self, id, text='', title='', css_class='', valid_for=TEMPLATE_TYPES, width=''):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        LabelComponent.__init__(self, id, text=text, title=title, css_class=css_class, valid_for=valid_for,
                                label_type=LABEL_TYPE_PAGE_TEMPLATE)
        self.width = width

InitializeClass(FieldColumnComponent)
componentClasses.append(FieldColumnComponent)

class ButtonComponent(AbstractComponent):
    meta_type = 'Button Component'
    icon = 'misc_/FormulatorLayout/ButtonComponent'
    security = ClassSecurityInfo()

    _properties = AbstractComponent._properties  + (
        {'id':'button_type', 'type':'selection', 'mode':'w', 'default':'button', 'select_variable':'getButtonTypes'},
        {'id':'action', 'type':'string', 'mode':'w','default':''}
        )

    def __init__(self, id, title='', css_class='', valid_for=TEMPLATE_TYPE_FORM, action='',
                 button_type=BUTTON_TYPE_SUBMIT):
        AbstractComponent.__init__(self, id, title, css_class, valid_for)
        self.action = action
        self.button_type = button_type

    _form_template = getTemplateFile('templates/ButtonComponent')
    
InitializeClass(ButtonComponent)
componentClasses.append(ButtonComponent)

class ErrorComponent(AbstractComponent):
    meta_type = 'Error Component'
    icon = 'misc_/FormulatorLayout/ErrorComponent'
    security = ClassSecurityInfo()
    
    def __init__(self, id, title='', css_class=None, field_ids=[]):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        # FIXME: ErrorComponent want's to handle templates...
        AbstractComponent.__init__(self, id, title=title, css_class=css_class, valid_for=TEMPLATE_TYPE_FORM)
        self.field_ids = field_ids
        
    security.declareProtected(view, 'getErrors')
    def getErrors(self):
        ''' get a list of errors valid for this ErrorComponent from REQUEST.
            return [], if no errors were found
        '''
        REQUEST = self.REQUEST
        error_value = REQUEST.get('error_value', None)
        errors = []
        if error_value and error_value.errors:
            errors = error_value.errors
        return errors
        
    security.declareProtected(view, 'getErrorForField')
    def getErrorForField(self, field_or_field_id, errors):
        """ from a list of error objects, 'errors',
            return the error for field 'field', or None 
            if no such error exists
        """
        field_id = getattr(field_or_field_id, 'id', field_or_field_id)
        for error in errors:
            if error.field_id == field_id:
                return error
        return None

    security.declarePrivate(view, 'getFieldIds')
    def getFieldIds(self):
        if self.field_ids:
            return self.field_ids
        else:
            return []
        
    security.declarePrivate('getBoundNames')
    def getBoundNames(self, **kw):
        errors = self.getErrors()
        field_ids = self.field_ids
        return {'errors': errors,
                'field_ids': field_ids}

    security.declarePrivate('isValidFor')
    def isValidFor(self, template_type):
        if template_type != TEMPLATE_TYPE_REPORT:
            if self.getErrors():
                return 1
        return 0
    
    _form_template = getTemplateFile('templates/ErrorComponent')
        
InitializeClass(ErrorComponent)
componentClasses.append(ErrorComponent)
    
class ErrorColumnComponent(FieldColumnComponent, ErrorComponent):
    meta_type = 'Error Column Component'
    icon = 'misc_/FormulatorLayout/ErrorComponent'
    security = ClassSecurityInfo()
    label_type = LABEL_TYPE_PAGE_TEMPLATE
    
    _properties = LabelComponent._properties + ({'id':'width', 'type':'string', 'mode':'w', 'default':''},)

    def __init__(self, id, text='', title='', css_class='', width=''):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        ErrorComponent.__init__(self, id, title=title,
                                css_class=css_class, field_ids=None)
        FieldColumnComponent.__init__(self, id, text=text, title=title, css_class=css_class,
                                      valid_for=TEMPLATE_TYPE_FORM,
                                      width=width)

    security.declarePrivate('getBoundNames')
    def getBoundNames(self, **kw):
        bound_names = FieldColumnComponent.getBoundNames(self, **kw)
        bound_names.update(ErrorComponent.getBoundNames(self, **kw))
        return bound_names

InitializeClass(ErrorColumnComponent)
componentClasses.append(ErrorColumnComponent)

